<?php

namespace Drupal\toolshed\Element;

use Drupal\Core\Render\Element;
use Drupal\Core\Render\Element\RenderElement;

/**
 * Create a button element that is a HTML button element.
 *
 * @RenderElement("toolshed_toolbar")
 */
class Toolbar extends RenderElement {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    return [
      '#pre_render' => [static::class . '::preRenderToolbar'],
    ];
  }

  /**
   * Format and prepare the toolbar for rendering.
   *
   * @param array $element
   *   Renderable toolbar panes and button information.
   *
   * @return array
   *   Processed and renderable elements of the toolbar. The toolbar buttons
   *   and panes organized together.
   */
  public static function preRenderToolbar(array $element) {
    $element['#attached']['library'][] = 'toolshed/Toolbar';

    foreach (Element::children($element['panes']) as $child) {
      $pane = &$element['panes'][$child];
      $pane += [
        '#type' => 'details',
        '#title' => 'selection',
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      ];

      if (empty($pane['#attributes']['class'])) {
        $pane['#attributes']['class'] = [];
      }

      $pane['#attributes']['class'][] = 'toolbar-pane';
      $pane['#attributes']['class'][] = "toolbar-pane--{$key}";
    }

    return $element;
  }

}
