<?php

namespace Drupal\toolshed\Plugin;

use Drupal\Component\Plugin\Exception\PluginNotFoundException;

/**
 * Create a plugin manager that maintains single instances of plugins.
 *
 * Most YAML discovered plugins are built from the exact same configuration
 * and definitions. These when these plugins are usually providing a service or
 * definition which are all meant to be single instances, this plugin
 * manager can be utilized to always retrieve and return the single instance.
 *
 * These definitions should have no external configurations, and only require
 * the $plugin_definition (YAML discovery) to get created.
 */
class StaticYamlPluginManager extends YamlPluginManager {

  /**
   * All loaded instances based on plugin ID.
   *
   * @var \Drupal\Component\Plugin\PluginInspectionInterface[]
   */
  protected static $instances = [];

  /**
   * {@inheritdoc}
   */
  public function createInstance($plugin_id, array $configuration = []) {
    if (!isset(static::$instances[$plugin_id])) {
      static::$instances[$plugin_id] = parent::createInstance($plugin_id);
    }

    return static::$instances[$plugin_id];
  }

  /**
   * {@inheritdoc}
   */
  public function getInstance(array $options) {
    if (!empty($options['id'])) {
      try {
        return $this->createInstance($options['id']);
      }
      catch (PluginNotFoundException $e) {
        return FALSE;
      }
    }

    return FALSE;
  }

}
